function [beta_k, Pk]=CSKF_p(beta_k0, Pk0, xb, Hk, Rk, Re, n_iterations, q)

% The CS-embedded Kalman filter (l_p norm variant). For more details see
% 'Methods for Sparse Signal Recovery Using Kalman Filtering with Embedded
% Pseudo-Measurement Norms and Quasi-Norms' by A. Carmi, P. Gurfil and D.
% Kanevsky, IEEE TRANSACTIONS ON SIGNAL PROCESSING, VOL. 58, NO. 4, APRIL 2010


m=size(Hk,2);

Pk=Pk0;
beta_k=beta_k0;

Kk=Pk*Hk'*inv(Hk*Pk*Hk'+Rk);
beta_k=beta_k+Kk*(xb-Hk*beta_k);
Pk=(eye(m)-Kk*Hk)*Pk*(eye(m)-Kk*Hk)'+Kk*Rk*Kk';

for j=1:n_iterations,
    
    s1=sum(abs(beta_k).^q);
    
    z=s1^(1/q);
    
    for i=1:m,
        if (beta_k(i) > 0)
            xa1(i)=s1^(1/q-1)*beta_k(i)^(q-1);
        else
            xa1(i)=-s1^(1/q-1)*(-beta_k(i))^(q-1);
        end
    end
    
    Hk=xa1;
    
    Kk=Pk*Hk'/(Hk*Pk*Hk'+Re);
    beta_k=beta_k-Kk*z;
    
    % The covariance update of the pseudo-measurement stage is not
    % mandatory. The algorithm runs faster when this stage is omitted.
    
    %   Pk=(eye(m)-Kk*Hk)*Pk*(eye(m)-Kk*Hk)'+Kk*Re*Kk';
end
